@@ -18,12 +18,12 @@ import sphinx_rtd_theme
 
 # -- Project information -----------------------------------------------------
 
-project = 'pspman'
+project = 'PSPMan'
 copyright = '2021, Pradyumna Paranjape'
 author = 'Pradyumna Paranjape'
 
 # The full version, including alpha/beta/rc tags
-release = '1!1.2.0'
+release = '1!1.3.0'
 
 
 # -- General configuration ---------------------------------------------------
