@@ -2,7 +2,7 @@
 # -*- coding:utf-8; mode:python -*-
 #
 # Copyright 2020 Pradyumna Paranjape
-# This le is part of pspman.
+# This file is part of pspman.
 #
 # pspman is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -24,14 +24,14 @@ PSPMAN: **PS**\ eudo **P**\ ackage **Man**\ ager
 
 
 # Globally defined environment configuration
-import os
+from pathlib import Path
 from psprint import init_print
 from .classes import InstallEnv
 from .config import read_config
 
 
-print = init_print(os.path.join(os.path.dirname(__file__),
-                                ".psprintrc.yml")).psprint
+print = init_print(Path(__file__).resolve()
+                   .parent.joinpath(".psprintrc.yml")).psprint
 '''
 Customized psprint function
 
@@ -44,7 +44,6 @@ CONFIG = read_config()
 Meta data information about C_DIR(s), configuration directory, etc
 '''
 
-
 # set configurations
 ENV = InstallEnv(CONFIG)
 '''
@@ -52,4 +51,4 @@ Standard installation context
 '''
 
 
-__version__ = '1!1.2.0'
+__version__ = '1!1.3.0'
