@@ -2,7 +2,7 @@
 # -*- coding:utf-8; mode:python -*-
 #
 # Copyright 2020 Pradyumna Paranjape
-# This le is part of pspman.
+# This file is part of pspman.
 #
 # pspman is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -28,6 +28,7 @@ from .psp_in import init, de_init
 from .config import GroupDB
 from . import ENV, CONFIG, print, __version__
 from .define import cli_opts, prepare_env, lock
+from .installations import INST_METHODS
 from .serial_actions import (interrupt, find_gits, end_queues, init_queues,
                              del_projects, add_projects, print_projects,
                              update_projects, print_prefixes)
@@ -46,39 +47,48 @@ def call() -> int:
         Error code to system
 
     '''
-    env = ENV.update(cli_opts(CONFIG))
+    cli_kwargs = cli_opts(CONFIG)
+    call_function = cli_kwargs.get('call_function')
 
-    if env.call_function == 'goodbye':
+    if call_function == 'goodbye':
         de_init(CONFIG)
         return 0
 
-    if env.call_function == 'init':
-        err = init(CONFIG)
+    if call_function == 'init':
+        opt_in = [i_t
+                  for d_i in INST_METHODS.values()
+                  for i_t in d_i.instruct.requires
+                  if i_t not in (cli_kwargs.get('ignore') or [])]
+        err = init(CONFIG, opt_in=opt_in)
         if err:
             return err
-        default_git_grp = GroupDB(path=CONFIG.data_dir, name='default')
+        # Build meta
+        CONFIG.opt_in = opt_in
+        default_git_grp = GroupDB(grp_path=CONFIG.data_dir, name='default')
         CONFIG.add(default_git_grp)
         CONFIG.store()
         default_git_grp.mk_structure()
         print("Close this shell (terminal) and open another session.")
         return 0
 
+    if call_function == 'version':
+        print(__version__, mark='info', pref='VERSION')
+        return 0
+
     if 'default' not in CONFIG.meta_db_dirs:
         # not initiated
         return 1
 
-    if env.call_function == 'version':
-        print(__version__, mark='info', pref='VERSION')
-        return 0
+    env = ENV.update(cli_kwargs)
 
-    if env.call_function == 'meta':
+    if call_function == 'meta':
         return print_prefixes(env=env)
 
     env_err = prepare_env(env)
     if env_err != 0:
         return env_err
 
-    lock_state = lock(env=env, unlock=(env.call_function == 'unlock'))
+    lock_state = lock(env=env, unlock=(call_function == 'unlock'))
     if lock_state != 0:
         return lock_state - 1
 
@@ -115,7 +125,7 @@ def call() -> int:
         end_queues(env=env, queues=queues)
         lock(env=env, unlock=True)
         print()
-        CONFIG.add({'path': env.prefix})
+        CONFIG.add({'grp_path': env.prefix})
         CONFIG.prune()
         CONFIG.store()
         print('done.', mark=1)
