@@ -2,7 +2,7 @@
 # -*- coding:utf-8; mode:python -*-
 #
 # Copyright 2020 Pradyumna Paranjape
-# This le is part of pspman.
+# This file is part of pspman.
 #
 # pspman is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -22,28 +22,16 @@ Exceptions, Warnings, Errors
 '''
 
 
+import os
+import typing
+
+
 class PSPManError(Exception):
     '''
     base error for papman
     '''
 
 
-class TagError(PSPManError):
-    '''
-    Error in tagging a git for type of modification (pull, install, etc)
-
-    Args:
-        old_tag: existing tag
-        operate: operation intended
-
-    '''
-
-    def __init__(self, old_tag: int, operate: str) -> None:
-        super(PSPManError, self).__init__(f'''
-        Tag '{old_tag}' can't be operated by '{operate}'
-        ''')
-
-
 class ClosedQueueError(PSPManError):
     '''
     Operation can't be performed, since the queue has been closed
@@ -53,19 +41,15 @@ class ClosedQueueError(PSPManError):
 
     '''
     def __init__(self, queue):
-        super(PSPManError, self).__init__('''
-        {type(queue)} Queue is closed
-        ''')
+        super().__init__(self, f'{type(queue)} Queue is closed')
 
 
 class PathNameError(PSPManError):
     '''
     Path/URL (and hence or otherwise, name) can't be determined
     '''
-    def __init__(self):
-        super(PSPManError, self).__init__(f'''
-        Path and/name not found/inferred
-        ''')
+    def __init__(self, *args):
+        super().__init__(self, 'Path and/name not found/inferred', *args)
 
 
 class GitURLError(PathNameError):
@@ -73,9 +57,7 @@ class GitURLError(PathNameError):
     Git URL (and hence or otherwise, name) can't be determined
     '''
     def __init__(self):
-        super(PSPManError, self).__init__(f'''
-        GIT URL and/name not found/inferred
-        ''')
+        super().__init__(self, 'GIT URL and/name not found/inferred')
 
 
 class CommandError(PSPManError):
@@ -95,3 +77,47 @@ class CommandError(PSPManError):
         STDERR from command:
         {err}
         ''')
+
+
+class InstructError(PSPManError):
+    '''
+    Base class for instruction files error
+    '''
+
+
+class InstructFmtError(InstructError):
+    '''
+    Error parsing installation instructions file
+
+    Args:
+        instruct_file: name of file with fault
+    '''
+    def __init__(self, instruct_file: typing.Union[str, os.PathLike]) -> None:
+        super().__init__(self, f'{instruct_file} is incorrectly formatted')
+
+
+class MissingInstructError(InstructError):
+    '''
+    Error in installation instructions file: necessary instruction missing
+
+    Args:
+        missing: name of argument missing
+    '''
+    def __init__(self, missing: str) -> None:
+        super().__init__(self, f'missing instruction: {missing}')
+
+
+class InstructTypeError(TypeError, InstructError):
+    '''
+    Wrong type of instruction
+
+    Args:
+        key: variable with bad type
+        found_type: type of supplied variable
+        want_type: type expected
+
+    '''
+    def __init__(self, key: str, found_type: type, want_type: str):
+        super().__init__(
+            self, f'{key}: is of type: {found_type}; expected: {want_type}'
+        )
