@@ -2,7 +2,7 @@
 # -*- coding:utf-8; mode:python -*-
 #
 # Copyright 2020 Pradyumna Paranjape
-# This le is part of pspman.
+# This file is part of pspman.
 #
 # pspman is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -25,9 +25,8 @@ Read configuration
 
 import os
 import typing
-import subprocess
+from pathlib import Path
 import shutil
-import yaml
 from .config import MetaConfig
 from .errors import PathNameError
 
@@ -35,26 +34,27 @@ from .errors import PathNameError
 _PSPMAN_MARKERS = "### PSPMAN MOD ###"
 
 
-def _sys_bash(path: str = None) -> str:
+def _sys_bash(sys_bash: typing.Union[str, os.PathLike] = None) -> Path:
     '''
-    System-specific path
+    System-specific bashrc path
 
     Args:
-        path: assert that path exists and return
+        sys_bash: assert that it exists and return
 
     Returns:
         Path to system-bashrc
 
     '''
-    if path:
-        if os.path.isfile(path):
-            return path
+    if sys_bash:
+        bash_path = Path(sys_bash)
+        if bash_path.is_file():
+            return bash_path.resolve()
         else:
             raise FileNotFoundError('Custom supplied bashrc file not found')
     if 'HOME' not in os.environ:
         raise FileNotFoundError('Default bashrc file not found')
-    path = os.path.join(os.environ['HOME'], ".bashrc")
-    return path
+    bash_path = Path(os.environ['HOME']).joinpath(".bashrc")
+    return bash_path.resolve()
 
 
 def _gen_sys_bash_text(psp_bash: str) -> str:
@@ -65,22 +65,34 @@ def _gen_sys_bash_text(psp_bash: str) -> str:
         psp_bash: path to config_dir/pspman/bashrc file
 
     '''
+    if 'HOME' in os.environ:
+        home_dir = Path(os.environ['HOME']).resolve()
+        # Path relative to home, since this is personal
+        # Such .bashrc can be synced between different users on same machine
+        psp_bash = "${HOME}/" + str(Path(psp_bash)
+                                    .resolve().relative_to(home_dir))
+    shell_check_source = psp_bash.replace("${HOME}/", '')
     return "\n".join(('', _PSPMAN_MARKERS,
-                           f'if [[ -f "{psp_bash}" ]]; then',
-                           f'    . "{psp_bash}"; fi',
-                           _PSPMAN_MARKERS, ''))
+                      f'# shellcheck source="{shell_check_source}"',
+                      f'if [[ -f "{psp_bash}" ]]; then',
+                      '    # shellcheck disable=SC1091',
+                      f'    . "{psp_bash}"; fi',
+                      _PSPMAN_MARKERS, ''))
 
 
-def _gen_psp_bash_text(data_path: str) -> str:
+def _gen_psp_bash_text(data: Path) -> str:
     '''
     Generate text for bashrc modifier
 
+    Args:
+        data: path to data folder
+
     '''
-    bin_path = os.path.join(data_path, 'bin')
-    py_path = os.path.join(data_path, 'lib', '${python_ver}', 'site-packages')
+    bin_path = data.joinpath('bin')
+    py_path = data.joinpath('lib', '${python_ver}', 'site-packages')
     rc_text = (
         '# shellcheck shell=bash',
-        "#-*- coding:utf-8; mode: shell-script -*-",
+        '#-*- coding:utf-8; mode: shell-script -*-',
         '',
         "python_ver=\"$(python3 --version " +
         "| cut -d '.' -f1,2 " +
@@ -103,7 +115,7 @@ def _gen_psp_bash_text(data_path: str) -> str:
     return "\n".join(rc_text)
 
 
-def _init_bash(psp_bash: str, data_path: str):
+def _init_bash(psp_bash: Path, data_path: Path):
     '''
     Initiate bash rc
 
@@ -116,7 +128,8 @@ def _init_bash(psp_bash: str, data_path: str):
         bashrc.write(_gen_psp_bash_text(data_path))
 
 
-def mod_bash(psp_bash: str, sys_bash: str = None):
+def mod_bash(psp_bash: typing.Union[str, os.PathLike],
+             sys_bash: typing.Union[str, os.PathLike] = None):
     '''
     Modify user's bashrc to include a reference to psp_bash
 
@@ -124,21 +137,22 @@ def mod_bash(psp_bash: str, sys_bash: str = None):
         psp_bash: path to config_dir/pspman/bashrc file
         sys_bash: path to default bashrc
     '''
-    mod_text = _gen_sys_bash_text(psp_bash)
+    mod_text = _gen_sys_bash_text(str(psp_bash))
     sys_bash = _sys_bash(sys_bash)
-    if os.path.isfile(sys_bash):
+    if sys_bash.is_file():
         with open(sys_bash, 'r') as bashrc_h:
             rc_text = bashrc_h.read()
             if mod_text in rc_text:
                 # already exported
                 return
-    if os.path.isdir(sys_bash):
+    if sys_bash.is_dir():
         raise IsADirectoryError(f'{sys_bash} should be a file, is a directory')
     with open(sys_bash, 'a') as bashrc_h:
         bashrc_h.write(mod_text)
 
 
-def restore_bash(psp_bash: str, sys_bash: str = None) -> None:
+def restore_bash(psp_bash: typing.Union[str, os.PathLike],
+                 sys_bash: typing.Union[str, os.PathLike] = None) -> None:
     '''
     Restore bashrc
 
@@ -147,8 +161,8 @@ def restore_bash(psp_bash: str, sys_bash: str = None) -> None:
         psp_bash: path to config_dir/pspman/bashrc file
 
     '''
-    sys_bash =  _sys_bash(sys_bash)
-    alteration = _gen_sys_bash_text(psp_bash)
+    sys_bash = _sys_bash(sys_bash)
+    alteration = _gen_sys_bash_text(str(psp_bash))
     with open(sys_bash, 'r') as bashrc_h:
         rc_text = bashrc_h.read()
     if alteration not in rc_text:
@@ -159,7 +173,8 @@ def restore_bash(psp_bash: str, sys_bash: str = None) -> None:
         bashrc_h.write(rc_text)
 
 
-def de_init(config: MetaConfig, sys_bash: str = None):
+def de_init(config: MetaConfig,
+            sys_bash: typing.Union[str, os.PathLike] = None):
     '''
     Undo changes made by init
 
@@ -169,7 +184,7 @@ def de_init(config: MetaConfig, sys_bash: str = None):
 
     '''
     sys_bash = _sys_bash(sys_bash)
-    psp_bash = os.path.join(config.config_dir, 'bashrc')
+    psp_bash = config.config_dir.joinpath('bashrc')
     restore_bash(psp_bash, sys_bash)
     instructions = [
         '',
@@ -184,8 +199,9 @@ def de_init(config: MetaConfig, sys_bash: str = None):
         instructions.append("[\033[1;31;40mNone\033[0m]")
     else:
         for group_name, group_db in config.meta_db_dirs.items():
-            instructions.append('# \033[0;94;40m' + 'rm -rf ' + group_db.path +
-                                '\033[0m  # ' + group_name)
+            instructions.append('# \033[0;94;40m' + 'rm -rf ' +
+                                str(group_db.grp_path) + '\033[0m  # ' +
+                                group_name)
 
     instructions.extend([
         '',
@@ -197,30 +213,32 @@ def de_init(config: MetaConfig, sys_bash: str = None):
     print('\n    '.join(instructions))
 
 
-def init(config: MetaConfig):
+def init(config: MetaConfig, opt_in: typing.List[str] = None):
     '''
     Initialize environment, data structure for pspman
 
     Args:
         config: OS-specific configuration initialized by pspman
 
-'''
+    '''
 
-    psp_config = os.path.join(config.config_dir, 'config.yml')
-    psp_bash = os.path.join(config.config_dir, 'bashrc')
+    opt_in = opt_in or []
+    psp_config = config.config_dir.joinpath('config.yml')
+    psp_bash = config.config_dir.joinpath('bashrc')
     mod_bash(psp_bash)
-    if not os.path.isfile(psp_bash):
-        if os.path.isdir(psp_bash):
+    if not psp_bash.is_file():
+        if psp_bash.is_dir():
             # Thou Shallt be a file
             shutil.rmtree(psp_bash)
     _init_bash(psp_bash, config.data_dir)
 
-    if not os.path.isfile(psp_config):
-        if os.path.isdir(psp_config):
+    if not psp_config.is_file():
+        if psp_config.is_dir():
             # Thou Shallt be a file
             shutil.rmtree(psp_config)
     dep_fail = False
-    for dependency in 'python3', 'pip', 'git' , 'make', 'cmake', 'meson', 'go':
+
+    for dependency in 'python3', 'git', 'pip', *opt_in:
         avail = shutil.which(dependency)
         if avail is None:
             print(f"Dependency not found: {dependency}")
